@@ -1,7 +1,7 @@
 import argparse
 
 import lazycon
-
+from pathlib import Path
 
 def build():
     parser = argparse.ArgumentParser('Build an experiment layout from the provided config.', add_help=False)
@@ -20,10 +20,12 @@ def run():
     parser = argparse.ArgumentParser('Run an experiment based on the provided config.', add_help=False)
     parser.add_argument('config')
     args = parser.parse_known_args()[0]
+    config_path = Path(args.config).absolute()
 
-    layout = lazycon.load(args.config).layout
+    layout = lazycon.load(config_path).layout
     layout.run_parser(parser)
     parser.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS, help='Show this message and exit')
 
     args = parser.parse_args()
-    layout.run(**vars(args))
+    folds = args.folds
+    layout.run(config=config_path, folds=folds)
